#ifndef DICTIONNAIRE_H
#define DICTIONNAIRE_H

typedef enum DIC_CONTEXT { DIC_GLOBAL=1, DIC_LOCAL, DIC_ARGUMENTS} DICContext;

typedef enum SYM_TYPE {SYM_BOOLEAN, SYM_REAL, SYM_INTEGER, SYM_STRING, SYM_VOID}SYMType;

typedef enum SYM_CLASS {CLASS_VARIABLE, CLASS_PROCEDURE, CLASS_ARRAY}SYMClass;

typedef enum ARG_TYPE{ARG_VALUE, ARG_REFERENCE}ARGType;

typedef struct BORNES{
    int BorneInf;
    int BorneSup;
}Bornes;

typedef struct PAIREBORNES
{
    Bornes bornes;
    struct PAIREBORNES *next;
}PaireBornes;

typedef union{
    ARGType ref;
    Bornes *bornes;
    int nArguments;
}Infos ;

typedef struct DICELEM {
    char *name;
    int id;
    int sp_offset;
    SYMType type;
    SYMClass classe;
    Infos infos;
    struct DICELEM *procArgList;
    struct DICELEM *next;
}DicElem;


/**************************************** Interface utilisateur **************************************************/

//choisit le contexte courant
void dicSetContext(DICContext);

//renvoie le contexte courant
DICContext dicGetContext();

//definit le type et la classe de la prochaine entree pour le constructeur
void dicSetBuilderType(SYMType);
void dicSetBuilderClass(SYMClass);
void dicSetBuilderOffset(int);
void dicSetBuilderRef(ARGType);

//change le type et la classe de l'entree courante
void dicSetType(SYMType);
void dicSetClass(SYMClass);

//recupere l'information indique de l'entree courante
SYMType dicGetType();
SYMClass dicGetClass();
int dicGetId();
char* dicGetName();

//ajoute une nouvelle entree au dictionnaire de type et de classe
// definis dans le constructeur
void dicAddEntry(char *name);

//ajoute une paire de bornes au tableau de l'entree courante
void dicSetArray(int,int);

//renvoie un pointeur sur la dim-ieme paire de bornes
//Bornes* dicGetArray(int);
Bornes* dicGetArray();

//force NULL a la liste des bornes
void dicVoidArray();

//recherche l'existence du nom, renvoie 1 si il existe dans le contexte courant,
// 0 sinon
int dicCheckExisting(char *);

//rend courante l'entree du nom recherche dans le contexte courant,
// renvoie 0 si non trouve
int dicSelect(char *);

//definit si l'entree courant est un passage par valeur ou par reference 
//(par default: ARG_VALUE en global et ARG_REFERENCE en local)
void dicSetRef(ARGType);

//renvoie le type de passage de la variable
ARGType dicGetRef();

//incremente le nombre d'arguments de la procedure courante
void dicIncArgsQuantity();

//recupere le nombre d'arguments
int dicGetArgsQuantity();

//efface le dictionnaire local 
void dicCleanLocal();

//modifie ou recupere l'offset par rapport au sommet de pile
void dicSetOffset(int);
int dicGetOffset();

//fait comme son nom l'indique
//renvoie 0 si inexistant, et ne changera rien
int dicSelectFirst();
int dicSelectNext();

//charge la liste d'arguments de la procedure selectionnee dans le contexte
//DIC_ARGUMENTS
void dicLoadArgList();

//grand nettoyage de printemps, on vire tout partout
void dicMrProper();

//renvoie 0 si n'existe pas, le numero du contexte
//ou il a ete trouve sinon
//ne change pas le contexte courant
DICContext dicSelectReachable (char *nom);

//renvoie 0 si n'existe pas dans tout le dictionnaire
//1 sinon
//ne change pas le contexte courant
int dicCheckExistingAtAll (char *nom);

/*selectionne le nArg'ieme argument de 'procedure'
  et reste dans le contexte DIC_ARGUMENTS
*/
void dicSelectArgument (int nArg, char *procedure);

#endif
